/**
 * 
 */
package org.enhydra.dm.business;

import java.io.IOException;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.DocumentVersion;
import org.enhydra.dm.api.FoDocument;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.business.exceptions.DatabaseException;
import org.enhydra.dm.data.DocumentDO;
import org.enhydra.dm.data.FoDocumentDO;
import org.enhydra.dm.util.Numerator;

/**
 * Implementation of Document interface.
 * 
 * @author Slobodan Vujasinovic
 */
public class DocumentImpl implements Document {

   private static final DateFormat LAST_MODIFIED_FORMAT = new SimpleDateFormat("E, dd MMM yyyy HH:mm:ss z",
                                                                               Locale.US);

   private static final String DOCUMENT_NUMBER_COLUMN_NAME = "DOCUMENT";

   private static MessageDigest digest;

   String id;

   String number;

   String name;

   String filepath;

   long size;

   String mimeType;

   String createdBy = null;

   String lockedBy = null;

   boolean archived;

   boolean autoversionable;

   String currentVersionNumber;

   String checkedOutBy = null;

   long createdDate;

   String lastModifiedBy = null;

   long lastModifiedDate;

   String deletedBy = null;

   long deletedDate;

   boolean template;

   Document templateRef = null;

   FoDocument foDocumentRef = null;

   DocumentImpl() {

   }

   public DocumentImpl(DocumentDO documentDO) throws BaseException {
      try {
         setId(documentDO.get_Handle());
         setNumber(String.valueOf(documentDO.getLOGNUM()));
         setName(documentDO.getNAME());
         setFilepath(documentDO.getFILEPATH());
         setSize(documentDO.getSIZE().intValue());
         setMimeType(documentDO.getMIMETYPE());
         setCreatedDate(documentDO.getCREATEDDATE().getTime());
         setArchived(documentDO.getARCHIVED());
         setAutoversionable(documentDO.getAUTOVERSIONABLE());
         setCurrentVersionNumber(documentDO.getCURRENTVERSIONNUMBER().toString());
         setLastModifiedDate(documentDO.getLASTMODIFIEDDATE().getTime());
         setTemplate(documentDO.getISTEMPLATE());

         setCreatedBy(documentDO.getCREATEDBY());
         setCheckedOutBy(documentDO.getCHECKEDOUTBY());
         setLockedBy(documentDO.getLOCKEDBY());
         setLastModifiedBy(documentDO.getLASTMODIFIEDBY());
         setDeletedBy(documentDO.getDELETEDBY());

         /*
          * Check for possible null values
          */

         DocumentDO templateDocDO = documentDO.getTEMPLATEREF();
         if (templateDocDO != null) {
            setTemplateRef(new DocumentImpl(templateDocDO));
         }

         FoDocumentDO foDocDO = documentDO.getFODOCUMENTREF();
         if (foDocDO != null) {
            setFoDocumentRef(new FoDocumentImpl(foDocDO));
         }

         Timestamp delDate = documentDO.getDELETEDDATE();
         if (delDate != null) {
            setDeletedDate(delDate.getTime());
         }
      } catch (Exception e) {
         throw new DatabaseException(e);
      }
   }

   /**
    * Returns the entity tag for the specified resource. The returned string uniquely
    * identifies the current incarnation of the given resource.
    * 
    * @param file The resource whose entity tag is to be retrieved.
    * @return A <code>String</code> containing the entity tag for the resource.
    */
   public String getETag() {

      try {
         digest = MessageDigest.getInstance("MD5");

         String key = this.getName() + ":" + Long.toHexString(this.getLastModifiedDate());
         byte[] hashBytes = null;
         synchronized (digest) {
            hashBytes = digest.digest(key.getBytes("UTF-8"));
         }
         StringBuffer hash = new StringBuffer();
         int count = hashBytes.length;
         for (int i = 0; i < count; i++) {
            hash.append(Integer.toHexString((hashBytes[i] >> 4) & 0x0f));
            hash.append(Integer.toHexString(hashBytes[i] & 0x0f));
         }
         return "\"" + hash.toString() + "\"";
      } catch (IOException ex) {
         return null;
      } catch (NoSuchAlgorithmException e) {
         return null;
      }
   }

   /**
    * Formats a timestamp (representing milliseconds since the epoch) as used in the
    * WebDAV <code>getlastmodified</code> property.
    * 
    * @return A <code>String</code> containing the formatted result.
    */
   public String getLastModifiedFormated() {
      synchronized (LAST_MODIFIED_FORMAT) {
         return LAST_MODIFIED_FORMAT.format(new Date(this.getLastModifiedDate()));
      }
   }

   public DocumentDO createDO() throws BaseException {
      try {
         DocumentDO documentDO = DocumentDO.createVirgin();
         documentDO.setNAME(this.getName());
         documentDO.setFILEPATH(this.getFilepath());
         documentDO.setSIZE(new BigDecimal(this.getSize()));
         documentDO.setMIMETYPE(this.getMimeType());
         documentDO.setCREATEDDATE(new Timestamp(this.getCreatedDate()));
         documentDO.setDELETEDDATE(new Timestamp(this.getDeletedDate()));
         documentDO.setARCHIVED(this.isArchived());
         documentDO.setAUTOVERSIONABLE(this.isAutoversionable());
         documentDO.setCURRENTVERSIONNUMBER(new BigDecimal(this.getCurrentVersionNumber()));
         documentDO.setLASTMODIFIEDDATE(new Timestamp(this.getLastModifiedDate()));
         documentDO.setISTEMPLATE(this.isTemplate());
         documentDO.setLOGNUM(Numerator.getNext(DOCUMENT_NUMBER_COLUMN_NAME).intValue());

         return documentDO;
      } catch (Exception e) {
         throw new DatabaseException(e);
      }
   }

   public DocumentVersion createVersion() {
      DocumentVersion dv = (DocumentVersion) new DocumentVersionImpl();
      dv.setFilepath(getFilepath());
      dv.setLastModifiedDate(getLastModifiedDate());
      dv.setSize(getSize());
      dv.setFoDocumentRef(getFoDocumentRef());
      dv.setVersionDate(System.currentTimeMillis());
      dv.setVersionNumber(getCurrentVersionNumber());

      return dv;
   }

   public boolean isArchived() {
      return archived;
   }

   public void setArchived(boolean archived) {
      this.archived = archived;
   }

   public boolean isAutoversionable() {
      return autoversionable;
   }

   public void setAutoversionable(boolean autoversionable) {
      this.autoversionable = autoversionable;
   }

   public String getCheckedOutBy() {
      return checkedOutBy;
   }

   public void setCheckedOutBy(String checkedOutBy) {
      this.checkedOutBy = checkedOutBy;
   }

   public String getCreatedBy() {
      return createdBy;
   }

   public void setCreatedBy(String createdBy) {
      this.createdBy = createdBy;
   }

   public long getCreatedDate() {
      return createdDate;
   }

   public void setCreatedDate(long createdDate) {
      this.createdDate = createdDate;
   }

   public String getCurrentVersionNumber() {
      return currentVersionNumber;
   }

   public void setCurrentVersionNumber(String currentVersionNumber) {
      this.currentVersionNumber = currentVersionNumber;
   }

   public String getFilepath() {
      return filepath;
   }

   public void setFilepath(String filepath) {
      this.filepath = filepath;
   }

   public FoDocument getFoDocumentRef() {
      return foDocumentRef;
   }

   public void setFoDocumentRef(FoDocument foDocumentRef) {
      this.foDocumentRef = foDocumentRef;
   }

   public String getLastModifiedBy() {
      return lastModifiedBy;
   }

   public void setLastModifiedBy(String lastModifiedBy) {
      this.lastModifiedBy = lastModifiedBy;
   }

   public long getLastModifiedDate() {
      return lastModifiedDate;
   }

   public void setLastModifiedDate(long lastModifiedDate) {
      this.lastModifiedDate = lastModifiedDate;
   }

   public String getLockedBy() {
      return lockedBy;
   }

   public void setLockedBy(String lockedBy) {
      this.lockedBy = lockedBy;
   }

   public String getMimeType() {
      return mimeType;
   }

   public void setMimeType(String mimeType) {
      this.mimeType = mimeType;
   }

   public String getName() {
      return name;
   }

   public void setName(String name) {
      this.name = name;
   }

   public String getNumber() {
      return number;
   }

   public String getId() {
      return id;
   }

   public void setNumber(String number) {
      this.number = number;
   }

   public void setId(String id) {
      this.id = id;
   }

   public long getSize() {
      return size;
   }

   public void setSize(long size) {
      this.size = size;
   }

   public boolean isTemplate() {
      return template;
   }

   public void setTemplate(boolean template) {
      this.template = template;
   }

   public Document getTemplateRef() {
      return templateRef;
   }

   public void setTemplateRef(Document templateRef) {
      this.templateRef = templateRef;
   }

   public String getDeletedBy() {
      return deletedBy;
   }

   public void setDeletedBy(String deletedBy) {
      this.deletedBy = deletedBy;
   }

   public long getDeletedDate() {
      return deletedDate;
   }

   public void setDeletedDate(long deletedDate) {
      this.deletedDate = deletedDate;
   }

}
