package org.enhydra.dm.business;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Properties;

import org.enhydra.dm.api.DocumentStore;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.business.exceptions.ConfigurationException;
import org.enhydra.dm.business.exceptions.DocumentStoreException;

/**
 * Default implementation of DocumentStore interface. Document are stored in docstore
 * like: doctore/yyyy/mm/dd/timestamp.extension
 * 
 * @author Svjetlana Milidrag
 */
public class DocumentStoreImpl implements DocumentStore {

   private String docStorePath = null;

   private Log logger = null;

   public DocumentStoreImpl() {

   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#saveDocumentContent(java.lang.String, byte[])
    */
   public String saveDocumentContent(String documentName, byte[] documentContent)
      throws BaseException {

      if (null != logger) {
         logger.log(Log.DEBUG,
                    "Saving document content for document for the first time.)",
                    documentName);
      }
      FileOutputStream fos = null;
      try {

         File documentFile = new File(createDocumentStorePath(documentName));
         if (!documentFile.exists()) {
            documentFile.createNewFile();
         }
         fos = new FileOutputStream(documentFile);
         fos.write(documentContent);
         fos.flush();

         return documentFile.getCanonicalPath();
      } catch (IOException e) {

         throw new DocumentStoreException("IOException :" + e.getMessage());
      } finally {
         try {
            if (null != fos) {
               fos.close();
            }
         } catch (IOException ex) {
            throw new DocumentStoreException("IOException :" + ex.getMessage());
         }
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#saveDocumentContent(java.lang.String, byte[],
    *      java.lang.String)
    */
   public String saveDocumentContent(String documentName,
                                     byte[] documentContent,
                                     String path) throws BaseException {
      if (null != logger) {
         logger.log(Log.DEBUG, "Saving document content for document)", documentName);
      }
      FileOutputStream fos = null;
      try {

         File documentFile = new File(path);
         if (!documentFile.exists()) {
            documentFile.createNewFile();
         }
         fos = new FileOutputStream(documentFile);
         fos.write(documentContent);
         fos.flush();

         return documentFile.getCanonicalPath();
      } catch (IOException e) {
         throw new DocumentStoreException("IOException :" + e.getMessage());
      } finally {
         try {
            if (null != fos) {
               fos.close();
            }
         } catch (IOException ex) {
            throw new DocumentStoreException("IOException :" + ex.getMessage());
         }
      }
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#loadDocumentContent(java.lang.String)
    */
   public byte[] loadDocumentContent(String documentFilePath) throws BaseException {
      if (null != logger) {
         logger.log(Log.DEBUG, "Loading document content from document store)");
      }
      byte[] buf = new byte[1024];
      ByteArrayOutputStream baos = null;
      BufferedOutputStream bos = null;
      FileInputStream fis = null;

      File documentFile = new File(documentFilePath);
      if (!documentFile.exists()) {
         throw new DocumentStoreException("Document content is physically deleted for document "
                                          + documentFilePath + "!");
      }

      try {
         baos = new ByteArrayOutputStream();
         bos = new BufferedOutputStream(baos);
         fis = new FileInputStream(documentFile);

         int b = 0;
         while ((b = fis.read(buf, 0, buf.length)) != -1) {
            bos.write(buf, 0, b);
         }
         bos.flush();

         return baos.toByteArray();
      } catch (Exception ex) {
         throw new DocumentStoreException("Error while loading document content!");
      } finally {
         if (baos != null)
            try {
               baos.close();
            } catch (IOException e) {
               throw new DocumentStoreException("IOException :" + e.getMessage());
            }
         if (bos != null)
            try {
               bos.close();
            } catch (IOException e) {
               throw new DocumentStoreException("IOException :" + e.getMessage());
            }
         if (fis != null) {
            try {
               fis.close();
            } catch (IOException e) {
               throw new DocumentStoreException("IOException :" + e.getMessage());
            }
         }
      }

   }

   /**
    * Method for create docstore path : doctore/yyyy/mm/dd/timestamp.extension
    * 
    * @param documentName
    * @return
    */
   protected String createDocumentStorePath(String documentName) {
      Calendar calendar = Calendar.getInstance();

      String ret = docStorePath
                   + File.separator + String.valueOf(calendar.get(Calendar.YEAR))
                   + File.separator;

      int month = calendar.get(Calendar.MONTH) + 1;
      ret += ((month >= 10) ? String.valueOf(month) : "0" + String.valueOf(month));

      int day = calendar.get(Calendar.DAY_OF_MONTH);
      ret += File.separator
             + ((day >= 10) ? String.valueOf(day) : "0" + String.valueOf(day));
      File filePath = new File(ret);
      filePath.mkdirs();
      String extension = "";
      if (documentName.indexOf(".") != -1) {
         extension = documentName.substring(documentName.lastIndexOf("."));
      }
      documentName = calendar.getTimeInMillis() + extension;
      return ret + File.separator + documentName;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#getLogger()
    */
   public Log getLogger() {
      return logger;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#setLogger(org.enhydra.dm.api.loggers.Log)
    */
   public void setLogger(Log logger) {
      this.logger = logger;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#getDocStorePath()
    */
   public String getDocStorePath() {
      return docStorePath;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#setDocStorePath(java.lang.String)
    */
   public void setDocStorePath(String docStorePath) {
      this.docStorePath = docStorePath;
   }

   /*
    * (non-Javadoc)
    * 
    * @see org.enhydra.dm.api.DocumentStore#configure(java.util.Properties)
    */
   public void configure(Properties properties) throws BaseException {

      String documentStoreDefaultPath = System.getProperty("java.io.tmpdir");

      try {
         this.docStorePath = properties.getProperty("DocumentStore.path",
                                                    documentStoreDefaultPath);
         if (null == logger) {
            logger = Log.getInstance();
         }
      } catch (Exception e) {
         throw new ConfigurationException(e.getMessage());
      }
   }

}
