package org.enhydra.dm.api;

import java.util.Properties;

import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;

public interface DocumentManager {

   /**
    * Method to document checkout. This method accepts document id and user as Strings and
    * sets LOCK flag to true.
    * 
    * @param id (Document ID)
    * @param user (User ID)
    * @return
    * @throws BaseException - already checked out
    */
   public Document checkOut(String id, String user) throws BaseException;

   /**
    * Method to document unCheckout. This method accepts document id and user as Strings
    * and sets LOCK flag to false.
    * 
    * @param id
    * @param user
    * @throws BaseException - document isn't locked
    */
   public void unCheckOut(String id, String user) throws BaseException;

   /**
    * Method to document checkin. This method accepts document id, byte array document
    * content and user as Strings and sets LOCK flag to false.
    * 
    * @param id
    * @param documentPath
    * @param user
    * @throws BaseException - is document isn't checked out
    */
   public void checkIn(String id, String documentPath, String user) throws BaseException;

   /**
    * Method to document unlock. This method accepts document id and user as Strings and
    * sets LOCK flag to false.
    * 
    * @param id
    * @param user
    * @throws BaseException
    */
   public void unlock(String id, String user) throws BaseException;

   /**
    * Method to document lock. This method accepts document id and user as Strings. and
    * sets LOCK flag to false.
    * 
    * @param id
    * @param user
    * @throws BaseException
    */
   public void lock(String id, String user) throws BaseException;

   /**
    * Method to document update. This method will be executed after document save.
    * 
    * @param id
    * @param path
    * @param user
    * @param foDocumentId
    * @throws BaseException
    */
   public void update(String id, String path, String user, String foDocumentId)
      throws BaseException;

   /**
    * Method to get document from database by document id.
    * 
    * @param id
    * @return
    * @throws BaseException
    */
   public Document getDocument(String id) throws BaseException;

   /**
    * Method to get documentVersion from database by document version id.
    * 
    * @param id
    * @return
    * @throws BaseException
    */
   public DocumentVersion getDocumentVersion(String id) throws BaseException;

   /**
    * Method to delete document by document id. This method sets deleted flag to true and
    * lastModifiedBy to user
    * 
    * @param id
    * @param user
    * @throws BaseException
    */

   public void delete(String id, String user) throws BaseException;

   /**
    * Method that creates document in database.
    * 
    * @param documentName
    * @param documentPath
    * @param mimeType
    * @param user
    * @param foDocumentId
    * @param isTemplate
    * @param getTemplateRef
    * @return id
    * @throws BaseException
    */
   public String create(String documentName,
                        String documentPath,
                        String mimeType,
                        String user,
                        String foDocumentId,
                        boolean isTemplate,
                        String templateId) throws BaseException;

   /**
    * Getting document versions by document id.
    * 
    * @param id
    * @return
    * @throws BaseException
    */
   public DocumentVersion[] getDocumentVersions(String id) throws BaseException;

   /**
    * Getting all available documents or templates.
    * 
    * @param isTemplate
    * @return
    * @throws BaseException
    */
   public Document[] getDocuments(boolean isTemplate) throws BaseException;

   /**
    * Getting all deleted documents or templates.
    * 
    * @param isTemplate
    * @return
    * @throws BaseException
    */
   public Document[] getDeleted(boolean isTemplate) throws BaseException;

   /**
    * Restore document version - set as current version.
    * 
    * @param documentVersion
    * @param user
    * @throws BaseException
    */
   public void restoreDocumentVersion(String versionId, String user) throws BaseException;

   /**
    * Restore document version - set as current version.
    * 
    * @param documentVersion
    * @param user
    * @throws BaseException
    */
   public void restore(String id, String user) throws BaseException;

   /**
    * Autoversionable On/Off
    * 
    * @param documentId
    * @param user
    * @throws BaseException
    */
   public void switchAutoVersionable(String documentId, String user) throws BaseException;

   /**
    * Archived On/Off
    * 
    * @param documentId
    * @param user
    * @throws BaseException
    */
   public void switchArchived(String documentId, String user) throws BaseException;

   /**
    * Creates version for documents that are not autoversionable
    * 
    * @param documentDO
    * @param userName
    * @return
    * @throws BaseException
    */
   public void createVersion(String documentId, String user) throws BaseException;

   /**
    * Getter for logger
    * 
    * @return
    */
   public Log getLogger();

   /**
    * Setter for logger
    * 
    * @param logger
    */
   public void setLogger(Log logger);

   /**
    * Configure method that read parameters from Properties
    * 
    * @param properties
    * @throws BaseException
    */
   public void configure(Properties properties) throws BaseException;

}
