/**
 * 
 */

package org.enhydra.dm.api.handler;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.DocumentManager;
import org.enhydra.dm.api.DocumentStore;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.api.util.DesEncription;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * An abstract implementation of the <code>MethodHandler</code> interface. This class
 * serves as a convenient basis for building method handlers. In addition to providing
 * basic <code>init</code> and <code>destroy</code> methods, several useful utility
 * methods are supplied.
 * 
 * @author Slobodan Vujasinovic
 */
public abstract class AbstractHandler implements MethodHandler {

   private ServletConfig config;

   private DocumentManager documentManager;

   private DocumentStore documentStore;

   private DesEncription desEncription;

   private Log logger;

   private String lockRequestTimeout;

   public static final String REQUEST_URI_CHARSET = "WebDavServlet.RequestUriCharset";

   public static final String DEFAULT_LOCK_TIMEOUT = "-1";

   public static final String SESSION_ATT_LOCKED = "locked";

   /**
    * Initializes the method handler. This implementation stores the provided
    * <code>ServletConfig</code> object and makes it available via the
    * <code>getServletConfig</code> method. Subclasses overriding this method should
    * start by invoking
    * <p>
    * <code>super.init(config);</code>
    * 
    * @param config a <code>ServletConfig</code> object containing the servlet's
    *           configuration and initialization parameters.
    * @throws ServletException If an error occurs during initialization.
    */
   public void init(ServletConfig config,
                    DocumentManager documentManager,
                    DocumentStore documentStore,
                    DesEncription desEncription,
                    Log logger) throws ServletException {
      this.config = config;
      this.documentManager = documentManager;
      this.documentStore = documentStore;
      this.desEncription = desEncription;
      this.logger = logger;
      this.lockRequestTimeout = config.getInitParameter("lockRequestTimeout");
      if (null == this.lockRequestTimeout) {
         this.lockRequestTimeout = DEFAULT_LOCK_TIMEOUT;
      }
   }

   /**
    * method that reads parameters from request, decode them and returns parameter Map
    * 
    * @param request
    * @return
    * @throws ServletException
    */
   public Map readParameters(HttpServletRequest request) throws ServletException {
      Map parameterMap = new HashMap();

      try {
         String parameters = null;
         if (this.desEncription != null) {
            parameters = request.getPathInfo();
            parameters = request.getParameter(EnhydraDMConstants.PARAM);

            if (parameters == null) {
               return parameterMap;
            }

            parameters = this.desEncription.decrypt(parameters);

         } else {
            try {
               parameters = request.getQueryString();

               if (parameters == null) {
                  return parameterMap;
               }

               parameters = URLDecoder.decode(parameters, EnhydraDMConstants.ENCODING);
            } catch (Exception ex) {
               ex.printStackTrace();
            }
         }

         StringTokenizer st = new StringTokenizer(parameters, "&");

         String token = null;
         String parameterName = null;
         String parameterValue = null;
         while (st.hasMoreTokens()) {
            token = st.nextToken();
            int equalIndex = token.indexOf("=");

            if (-1 < equalIndex && equalIndex < token.length()) {
               parameterName = token.substring(0, equalIndex);
               parameterValue = token.substring(equalIndex + 1);
               parameterMap.put(parameterName, parameterValue);
            }
         }
      } catch (BaseException e) {
         if (null != logger) {
            logger.log(Log.ERROR, "Error during parameters decrypt!");
         }
         throw new ServletException(e.getMessage());
      }

      return parameterMap;
   }

   /**
    * Destroy method
    */
   public void destroy() {
      config = null;
      documentManager = null;
      documentStore = null;
      logger = null;
   }

   /**
    * Returns the <code>ServletConfig</code> object that was provided to the
    * <code>init</code> method.
    * 
    * @return A <code>ServletConfig</code> object containing the servlet's configuration
    *         and initialization parameters.
    */
   protected ServletConfig getServletConfig() {
      return config;
   }

   /**
    * @return documentManager
    */
   protected DocumentManager getDocumentManager() {
      return documentManager;
   }

   /**
    * @return documentStore
    */
   protected DocumentStore getDocumentStore() {
      return documentStore;
   }

   /**
    * @return logger
    */
   public Log getLogger() {
      return logger;
   }

   /**
    * @param logger
    */
   public void setLogger(Log logger) {
      this.logger = logger;
   }

   /**
    * @return user
    */
   public String getUser(HttpServletRequest request) {
      String user = request.getRemoteUser();
      return user;
   }

   /**
    * 
    */
   public abstract void service(HttpServletRequest request, HttpServletResponse response)
      throws IOException,
         ServletException;

   protected String getAllowedMethods() {
      StringBuffer allow = new StringBuffer();

      allow.append("OPTIONS");
      allow.append(", HEAD");
      allow.append(", GET");
      allow.append(", POST");
      allow.append(", DELETE");
      allow.append(", PROPFIND");
      allow.append(", PROPPATCH");
      allow.append(", COPY");
      allow.append(", MOVE");
      allow.append(", PUT");
      allow.append(", MKCOL");

      allow.append(", LOCK");
      allow.append(", UNLOCK");

      allow.append(", VERSION-CONTROL");
      allow.append(", CHECKOUT");
      allow.append(", UNCHECKOUT");
      allow.append(", CHECKIN");
      allow.append(", REPORT");

      return allow.toString();
   }

   public String getLockRequestTimeout() {
      return lockRequestTimeout;
   }

   /**
    * Checks if a conditional request should apply. If the client specifies one or more
    * conditional cache headers ("<code>If-Match</code>", "<code>If-None-Match</code>", "<code>If-Modified-Since</code>",
    * or "<code>If-Unmodified-Since" -- "<code>If-Range</code>" is not
    * currently supported), this method will indicate whether the
    * request should be processed. 
    *
    * @param request The servlet request whose conditional cache headers
    * will be examined.
    * @param file The resource that is being examined.
    * @return An HTTP status code indicating the result.  This will be one of:
    * <ul>
    * <li><code>200</code> (<code>HttpServletResponse.SC_OK</code>) --
    * if the request should be serviced normally</li>
    * <li><code>304</code> (<code>HttpServletResponse.SC_NOT_MODIFIED</code>)
    * -- if the resource has not been modified</li>
    * <li><code>400</code>
    * (<code>HttpServletResponse.SC_BAD_REQUEST</code>) --
    * if the client has submitted a malformed conditional header</li>
    * <li><code>412</code>
    * (<code>HttpServletResponse.SC_PRECONDITION_FAILED</code>) --
    * if no matching entity was found, or the request should not proceed
    * based on the current lock status</li>
    * </ul>
    * @throws IOException If an error occurs while examining the resource.
    */
   protected int checkConditionalRequest(HttpServletRequest request, Document document)
      throws IOException {
      Enumeration values = request.getHeaders("If-None-Match");
      if (values.hasMoreElements()) {
         String etag = document.getETag();
         if (etag != null) {
            boolean match = false;
            do {
               String value = (String) values.nextElement();

               if ("*".equals(value) || etag.equals(value))
                  match = true;
            } while (!match && values.hasMoreElements());
            if (match) {

               long timestamp = request.getDateHeader("If-Modified-Since");

               if (timestamp == -1
                   || timestamp >= (document.getLastModifiedDate() / 1000 * 1000)) {

                  return HttpServletResponse.SC_NOT_MODIFIED;
               } else {

               }
            } else {

            }
         }
      } else {
         values = request.getHeaders("If-Match");
         if (values.hasMoreElements()) {
            String etag = document.getETag();
            if (etag == null) {

               return HttpServletResponse.SC_PRECONDITION_FAILED;
            }
            boolean match = false;
            do {
               String value = (String) values.nextElement();

               if ("*".equals(value) || etag.equals(value))
                  match = true;
            } while (!match && values.hasMoreElements());
            if (!match) {

               return HttpServletResponse.SC_PRECONDITION_FAILED;
            } else {

            }
         }
         long timestamp = request.getDateHeader("If-Unmodified-Since");

         if (timestamp != -1) {
            if ((document.getLastModifiedDate() / 1000 * 1000) > timestamp) {

               return HttpServletResponse.SC_PRECONDITION_FAILED;
            } else {

            }
         } else {
            timestamp = request.getDateHeader("If-Modified-Since");

            if (timestamp != -1
                && timestamp >= (document.getLastModifiedDate() / 1000 * 1000)) {

               return HttpServletResponse.SC_NOT_MODIFIED;
            } else {

            }
         }
      }
      return HttpServletResponse.SC_OK;
   }

}
