/**
 *
 */

package org.enhydra.dm.handler;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.DocumentVersion;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.handler.AbstractHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * Default implementation of a handler for requests using the HTTP GET method.
 * 
 * @author Slobodan Vujasinovic
 */
public class DefaultGetHandler extends AbstractHandler {

   /**
    * Services requests which use the HTTP GET method. This implementation retrieves the
    * content for non-collection resources, using the content type information. If the
    * specified file does not exist, a 404 (Not Found) error is sent to the client.
    * 
    * @param request The request being serviced.
    * @param response The servlet response.
    * @throws ServletException If an application error occurs.
    * @throws IOException If an IO error occurs while handling the request.
    */

   public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {

      long modified = 0;
      Document document = null;
      byte[] buf;
      int contentLength;
      try {

         Map map = this.readParameters(request);

         String mod = (String) map.get(EnhydraDMConstants.ACTION);

         if (null == mod) {
            if (null != getLogger()) {
               getLogger().log(Log.DEBUG,
                               "(GET)Action parameter isn't defined parameter does not exist.");
            }

            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            response.flushBuffer();
            return;
         }
         if (mod.equals(EnhydraDMConstants.ACTION_READ_VERSION)) {

            String vdID = (String) map.get(EnhydraDMConstants.VERSION_ID);

            if (null == vdID) {
               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG,
                                  "(GET)DocumentVersion  parameter id does not exist.");
               }

               response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
               response.flushBuffer();
               return;
            }

            if (null != getLogger()) {
               getLogger().log(Log.DEBUG,
                               "GET Request for document version (id)\"{0}\".",
                               vdID);
            }

            DocumentVersion dv = getDocumentManager().getDocumentVersion(vdID);

            if (null == dv) {
               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG,
                                  "(GET)DocumentVersion does not exist for id:",
                                  vdID);
               }

               response.sendError(HttpServletResponse.SC_NOT_FOUND);
               return;
            }
            document = dv.getDocument();
            modified = dv.getLastModifiedDate();
            buf = getDocumentStore().loadDocumentContent(dv.getFilepath());
            contentLength = (int) dv.getSize();

         } else {

            String id = (String) map.get(EnhydraDMConstants.DOCUMENT_ID);

            if (null == id) {
               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG, "(GET)Document id parameter does not exist.");
               }
               response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
               response.flushBuffer();
               return;
            }

            if (null != getLogger()) {
               getLogger().log(Log.DEBUG, "GET Request for document (id)\"{0}\".", id);
            }

            document = getDocumentManager().getDocument(id);

            if (null == document) {

               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG, "(GET)Document does not exist.");
               }

               response.sendError(HttpServletResponse.SC_NOT_FOUND);
               return;
            }

            modified = document.getLastModifiedDate();
            buf = getDocumentStore().loadDocumentContent(document.getFilepath());
            contentLength = (int) document.getSize();

         }

         response.setHeader(EnhydraDMConstants.HEAD_PRAGMA, "public");
         response.setHeader(EnhydraDMConstants.HEAD_CACHE_CONTROL, "max-age=0");

         if (modified != 0) {
            response.setHeader(EnhydraDMConstants.HEAD_LAST_MODIFIED,
                               document.getLastModifiedFormated());
         }

         String etag = document.getETag();
         if (etag != null)
            response.setHeader(EnhydraDMConstants.HEAD_ETAG, etag);

         int result = checkConditionalRequest(request, document);
         if (result != HttpServletResponse.SC_OK) {
            response.setStatus(result);
            response.flushBuffer();
            return;
         }
         String contentType = document.getMimeType();

         response.setContentType((contentType != null) ? contentType
                                                      : "application/octet-stream");

         response.setContentLength(contentLength);

         response.setHeader(EnhydraDMConstants.HEAD_ALLOW, getAllowedMethods());

         ServletOutputStream output = response.getOutputStream();

         output.write(buf, 0, buf.length);

         output.flush();

      } catch (BaseException e) {

         throw new ServletException(e.getMessage());
      }
   }
}
