/**
 * 
 */

package org.enhydra.dm.handler;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.DocumentVersion;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.handler.AbstractHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * Default implementation of a handler for requests using the HTTP HEAD method.
 * 
 * @author Slobodan Vujasinovic
 */
public class DefaultHeadHandler extends AbstractHandler {

   /**
    * Services requests which use the HTTP HEAD method. This implementation returns basic
    * information regarding the specified resource. <br>
    * If the specified file does not exist, a 404 (Not Found) error is sent to the client.
    * 
    * @param request The request being serviced.
    * @param response The servlet response.
    * @throws ServletException If an application error occurs.
    * @throws IOException If an IO error occurs while handling the request.
    */

   public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {

      Map map = this.readParameters(request);

      String mod = (String) map.get(EnhydraDMConstants.ACTION);

      if (null == mod) {
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG,
                            "(HEAD)Action parameter isn't defined parameter does not exist.");
         }

         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;
      }

      long modified = 0;
      Document document = null;
      int contentLength = 0;

      try {

         if (mod.equals(EnhydraDMConstants.ACTION_READ_VERSION)) {

            String vdOID = (String) map.get(EnhydraDMConstants.VERSION_ID);

            if (null == vdOID) {
               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG,
                                  "(HEAD)DocumentVersion id parameter does not exist.");
               }

               response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
               response.flushBuffer();
               return;
            }

            if (null != getLogger()) {
               getLogger().log(Log.DEBUG,
                               "HEAD Request for document version (id)\"{0}\".",
                               vdOID);
            }

            DocumentVersion dv = getDocumentManager().getDocumentVersion(vdOID);
            if (null == dv) {

               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG,
                                  "(HEAD)DocumentVersion does not exist for id.",
                                  vdOID);
               }

               response.sendError(HttpServletResponse.SC_NOT_FOUND);
               return;
            }
            document = dv.getDocument();
            modified = dv.getLastModifiedDate();
            contentLength = (int) dv.getSize();

         } else {
            String id = (String) map.get(EnhydraDMConstants.DOCUMENT_ID);

            if (null == id) {
               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG,
                                  "(HEAD)Document id parameter does not exist.");
               }
               response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
               response.flushBuffer();
               return;
            }

            if (null != getLogger()) {
               getLogger().log(Log.DEBUG, "HEAD Request for document (id)\"{0}\".", id);
            }
            document = getDocumentManager().getDocument(id);

            if (null == document) {

               if (null != getLogger()) {
                  getLogger().log(Log.DEBUG, "(HEAD)Document does not exist for id.", id);
               }

               response.sendError(HttpServletResponse.SC_NOT_FOUND);
               return;
            }
            modified = document.getLastModifiedDate();
            contentLength = (int) document.getSize();

         }

         String contentType = document.getMimeType();

         response.setContentType((contentType != null) ? contentType
                                                      : "application/octet-stream");

         if (modified != 0) {
            response.setHeader(EnhydraDMConstants.HEAD_LAST_MODIFIED,
                               document.getLastModifiedFormated());
         }
         String etag = document.getETag();
         if (etag != null)
            response.setHeader(EnhydraDMConstants.HEAD_ETAG, etag);

         response.setHeader(EnhydraDMConstants.HEAD_ALLOW, getAllowedMethods());
         response.setContentLength(contentLength);
         response.flushBuffer();

      } catch (BaseException e) {

         throw new ServletException(e.getMessage());
      }
   }
}
