/**
 * 
 */

package org.enhydra.dm.handler;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.handler.AbstractHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * Default implementation of a handler for requests using the WebDAV PROPFIND method.
 * 
 * @author Slobodan Vujasinovic
 */
public class DefaultPropfindHandler extends AbstractHandler {

   /**
    * Services requests which use the WebDAV PROPFIND method. This implementation builds
    * and returns an XML document containing an appropriate PROPFIND result. <br>
    * If the specified resource does not exist, a 404 (Not Found) error is sent to the
    * client. <br>
    * If the PROPFIND request is not properly formed, a 400 (Bad Request) error is sent to
    * the client.
    * 
    * @param request The request being serviced.
    * @param response The servlet response.
    * @throws ServletException If an application error occurs.
    * @throws IOException If an IO error occurs while handling the request.
    */

   public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {
      try {
         setResponse(request, response);
      } catch (BaseException e) {

         throw new ServletException(e.getMessage());
      }

   }

   private void setResponse(HttpServletRequest request, HttpServletResponse response)
      throws IOException,
         ServletException,
         BaseException {

      Map map = readParameters(request);
      
      /*
       * User is required for this operation!
       */
      String user = getUser(request);
      if (null == user) {
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "(PROPFIND)User not defined.");
         }
         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;

      }

      String mod = (String) map.get(EnhydraDMConstants.ACTION);
      if (null == mod) {
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG,
                            "(PROPFIND)Action parameter isn't defined parameter does not exist.");
         }

         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;
      }
      String id = (String) map.get(EnhydraDMConstants.DOCUMENT_ID);
      if (null == id) {

         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "(PROPFIND)Document id parameter does not exist.");
         }

         response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
         response.flushBuffer();
         return;
      }
      
      if (null != getLogger()) {
         getLogger().log(Log.DEBUG, "PROPFIND Request for document (id)\"{0}\".");
      }
      Document document = getDocumentManager().getDocument(id);

      if (null == document) {

         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "(PROPFIND)Document does not exist for id.", id);
         }

         response.sendError(HttpServletResponse.SC_NOT_FOUND);
         return;
      }
      String content = "<?xml version=\"1.0\" encoding=\"utf-8\" ?>"
                       + "<multistatus xmlns=\"DAV:\" xmlns:a=\"DAV:\" >";
      if (mod.equals(EnhydraDMConstants.ACTION_EDIT)) {

         content = content
                   + "<response>"
                   + "<href>" + document.getFilepath() + "</href>"
                   + "<propstat>" 
                   + "<status>HTTP/1.1 200 MultiStatus</status>"
                   + "<prop>"
                   + "<getcontenttype>" + document.getMimeType() + "</getcontenttype>" 
                   + "<resourcetype/>" 
                   + "<displayname>" + document.getName() + "</displayname>" 
                   + " <supportedlock>" 
                      + " <lockentry>"
                         + " <lockscope>" 
                            + "<exclusive/>" 
                         + " </lockscope>"
                         + " <locktype>" 
                            + " <write/>" 
                         + " </locktype>" 
                      + "</lockentry>"
                      + " <lockentry>"
                         + " <lockscope>" 
                            + "<shared/>" 
                            + " </lockscope>"
                         + " <locktype>" 
                            + " <write/>" 
                         + " </locktype>" 
                      + "</lockentry>"
                   + "</supportedlock>" 
                   + "<lockdiscovery>" 
                      + "<activelock>"
                         + "<locktype>" 
                            + " <write/>" 
                         + "</locktype>" 
                         + "<lockscope>"
                            + "<exclusive/>" 
                            + "<shared/>" 
                         + "</lockscope>" 
                         + "<depth>0</depth>"
                         + "<owner>" + user + "</owner>"
                         + "<timeout>Second-180</timeout>" 
                         + "<locktoken>" + document.getNumber() + "</locktoken>" 
                     + "</activelock>" 
                   + "</lockdiscovery>"
                   + "<getcontentlength w:dt=\"int\">" + document.getSize() + "</getcontentlength>" 
                   + "<getetag>" + document.getETag() + "</getetag>" 
                   + "</prop>" 
                   + "</propstat>" 
                   + "</response>"
                   + "</multistatus>";
      } else {
         content = content
                   + "<response>" + "<href></href>" + "<propstat>"
                   + "<status>HTTP/1.1 200 MultiStatus</status>" + "<prop>"
                   + "<a:isreadonly w:dt=\"boolean\">1</a:isreadonly>" + " </prop>"
                   + "</propstat>" + "</response>" + "</multistatus>";
      }

      response.getWriter().write(content);
      response.setStatus(SC_MULTISTATUS);
      response.setHeader(EnhydraDMConstants.HEAD_ALLOW, getAllowedMethods());
      response.setContentType("text/xml; charset=\"utf-8\"");
      response.flushBuffer();
   }
}
