/**
 * 
 */

package org.enhydra.dm.handler;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.enhydra.dm.api.Document;
import org.enhydra.dm.api.exceptions.BaseException;
import org.enhydra.dm.api.handler.AbstractHandler;
import org.enhydra.dm.api.loggers.Log;
import org.enhydra.dm.util.EnhydraDMConstants;

/**
 * Default implementation of a handler for requests using the HTTP PUT method.
 * 
 * @author Slobodan Vujasinovic
 */
public class DefaultPutHandler extends AbstractHandler {

   /**
    * Services requests which use the HTTP PUT method. This implementation uploads the
    * content to the specified location. <br>
    * If the content length is not specified, a 411 (Length Required) error is sent to the
    * client. <br>
    * If the resource exists and is a collection, a 405 (Method Not Allowed) error is sent
    * to the client. <br>
    * If the parent collection does not exist, a 409 (Conflict) error is sent to the
    * client.
    * 
    * @param request The request being serviced.
    * @param response The servlet response.
    * @throws ServletException If an application error occurs.
    * @throws IOException If an IO error occurs while handling the request.
    */

   public void service(HttpServletRequest request, HttpServletResponse response)
      throws ServletException,
         IOException {
 
      try {
         int length = request.getContentLength();
         if (length < 0) {
            response.sendError(HttpServletResponse.SC_LENGTH_REQUIRED);
            return;
         }

         Map map = this.readParameters(request);

         String id = (String) map.get(EnhydraDMConstants.DOCUMENT_ID);

         if (null == id) {
            if (null != getLogger()) {
               getLogger().log(Log.DEBUG, "(PUT)Document id parameter does not exist.");
            }
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            response.flushBuffer();
            return;
         }
         
         if (null == request.getSession(true).getAttribute(SESSION_ATT_LOCKED)) {
            if (null != getLogger()) {
               getLogger().log(Log.WARNING, "(PUT) Conflict - Session Expired.");
            }
            response.setStatus(HttpServletResponse.SC_CONFLICT);
            response.flushBuffer();
            return;
         }
         
         if (null != getLogger()) {
            getLogger().log(Log.DEBUG, "PUT Request for document (id)\"{0}\".", id);
         }
         Document doc = getDocumentManager().getDocument(id);

         if (null == doc) {

            if (null != getLogger()) {
               getLogger().log(Log.DEBUG, "(PUT)Document does not exist for id.", id);
            }

            response.sendError(HttpServletResponse.SC_NOT_FOUND);
            return;
         }

         InputStream input = request.getInputStream();
         ByteArrayOutputStream output = new ByteArrayOutputStream(request.getContentLength());
         byte[] buf = new byte[8192];
         int count;
         while ((count = input.read(buf)) != -1) {
            output.write(buf, 0, count);
         }
         byte[] documentContent = output.toByteArray();
         output.flush();
         output.close();

         String path = (String) request.getSession().getAttribute(id);

         if (path == null || "".equals(path)) {
            path = getDocumentStore()
               .saveDocumentContent(doc.getName(), documentContent);
            request.getSession().setAttribute(id, path);
         } else {
            String result = getDocumentStore()
               .saveDocumentContent(doc.getName(), documentContent, path);
            if (null == result) {
               if (null != getLogger()) {
                  getLogger().log(Log.ERROR, "PUT Request failure for document.");
               }

               response.setStatus(HttpServletResponse.SC_CONFLICT);
               return;
            }
         }

         response.setStatus(HttpServletResponse.SC_CREATED);
         response.setHeader(EnhydraDMConstants.HEAD_LOCATION, request.getPathTranslated());
         response.setHeader(EnhydraDMConstants.HEAD_ALLOW, getAllowedMethods());
         response.flushBuffer();
      } catch (BaseException e) {

         response.setStatus(HttpServletResponse.SC_METHOD_NOT_ALLOWED);
         throw new ServletException(e.getMessage());
      }
   }
}
