package org.enhydra.dm.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.apache.fop.pdf.PDFEncryptionParams;
import org.enhydra.dm.api.exceptions.BaseException;
import org.w3c.tidy.Tidy;

public class TransformUtility {
   
   private static final String TRANSFORUTILITY_PROP_PREFFIX = "TransformUtility.";

   private static final String TRANSFORUTILITY_PROP_TIDYCONFIG = "TidyConfig";
   
   private static final String TRANSFORUTILITY_PROP_FOPCONFIG = "FopConfig";
   
   private static final String TRANSFORUTILITY_PROP_FOPBASE = "FopBase";
   
   private static final String TRANSFORUTILITY_XSLT = "org/enhydra/dm/util/xhtml2fo.xsl";
   
   /**
    * <code>tidy.properties</code> used by Tidy parser.
    */
   private static String tidyConfig = null;

   private static String fopBase = null;

   private static FopFactory fopFactory = null;

   private static TransformerFactory transformerFactory = null;
   
   private static Templates templates = null;
   
   /**
    * Method finds fiels and folders on the file sistem and make transformers,
    * necessary for making pdf file from editor content.
    * 
    * @param properties
    * @throws BaseException
    */
   public static void configure(Properties properties) throws BaseException {
      try {
         tidyConfig = properties.getProperty(TRANSFORUTILITY_PROP_PREFFIX+TRANSFORUTILITY_PROP_TIDYCONFIG);
         
         transformerFactory = TransformerFactory.newInstance();
         templates = transformerFactory.newTemplates(new StreamSource(TransformUtility.class.getClassLoader().getResourceAsStream(TRANSFORUTILITY_XSLT)));

         
         fopBase = properties.getProperty(TRANSFORUTILITY_PROP_PREFFIX+TRANSFORUTILITY_PROP_FOPBASE);
         
         String fopConfig = properties.getProperty(TRANSFORUTILITY_PROP_PREFFIX+TRANSFORUTILITY_PROP_FOPCONFIG);
         fopFactory = FopFactory.newInstance();
         fopFactory.setUserConfig(fopConfig);

      } catch (Exception ex) {
         throw new BaseException("TransformUtility Initialization Problem!",ex);
      }
   }
   
   /**
    * Method make Tidy parser using <code>tidyConfig.properties</code>
    * 
    * @return <code>Tidy</code> parser
    */
   public static Tidy getTidy() {
      Tidy tidy = new Tidy();
      tidy.setConfigurationFromFile(tidyConfig);
      return tidy;
   }
   
   /**
    * Method get <code>Taransformer</code>
    * 
    * @return <code>Transformer</code> object
    * @throws BaseException
    */
   public static Transformer getTransformer() throws BaseException {
      try {
         return templates.newTransformer();
      } catch (TransformerException ex) {
         throw new BaseException("Not able to load xhtml2fo transformation!",ex);
      }
   }

   public static FopFactory getFopFactory() {
      return fopFactory;
   }

   public static String getFopBase() {
      return fopBase;
   }

   public static TransformerFactory getTransformerFactory() {
      return transformerFactory;
   }
   
   
   /**
    * Method transforms TinyMCE submission from HTML to XHTML format using JTidy utility
    * 
    * @param cont
    * @return XHTML format of editor content
    * @throws BaseException
    */
   public static String html2xhtml(String content) throws BaseException {
      byte[] cont;
      ByteArrayInputStream bais = null;
      ByteArrayOutputStream baos = null;

      try {
         cont = content.getBytes(EnhydraDMConstants.ENCODING);
         bais = new ByteArrayInputStream(cont);
         baos = new ByteArrayOutputStream();

         Tidy tidy = TransformUtility.getTidy();
         tidy.getDropFontTags();
         tidy.parse(bais, baos);

         baos.flush();
         cont = baos.toByteArray();

         content = new String(cont, EnhydraDMConstants.ENCODING);

         content = content.replace("<body>",
                                   "<body> <font face=\"times new roman,times\">");
         content = content.replace("</body>", "</font></body>");
      } catch (Exception ex) {
         content = null;
         throw new BaseException(ex);
      } finally {
         if (bais != null)
            try {
               bais.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
         if (baos != null) {
            try {
               baos.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
         }
      }
      return content;
   }
   
   
   /**
    * Method transforms XHTML content format to FO format using xhtml2fo (XSl) template
    * 
    * @param cont
    * @return FO format of editor content
    * @throws BaseException
    */
   public static String xhtml2fo(String content) throws BaseException {
      byte[] cont;
      ByteArrayInputStream bais = null;
      ByteArrayOutputStream baos = null;

      try {
         cont = content.getBytes(EnhydraDMConstants.ENCODING);
         bais = new ByteArrayInputStream(cont);
         baos = new ByteArrayOutputStream();

         Transformer foTransformer = getTransformer();
         StreamSource ss = new StreamSource(bais);
         StreamResult sr = new StreamResult(baos);
         foTransformer.transform(ss, sr);

         baos.flush();
         cont = baos.toByteArray();

         content = new String(cont, EnhydraDMConstants.ENCODING);

         int start = content.indexOf("<fo:block role=\"html:body\">");
         int end = content.lastIndexOf("</fo:block>");

         if (start > 0 && end > 0) {
            end += 11;
            content = content.substring(start, end);
         }

      } catch (Exception ex) {
         content = null;
         throw new BaseException(ex);
      } finally {
         if (bais != null)
            try {
               bais.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
         if (baos != null) {
            try {
               baos.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
         }
      }
      return content;
   }
   
   /**
    * Method embeds editor content and other "Document properties" fields in the same
    * String that represents input for final transformation.
    * 
    * @param foParams
    * @return XML data format
    */
   public static String params2xml(HashMap foParams) {
      String key = null;
      String value = null;
      String content = "<?xml version=\"1.0\" encoding=\"UTF-8\"?> "
                       + "<root xmlns:fo=\"http://www.w3.org/1999/XSL/Format\"> ";

      if (foParams != null) {
         Iterator entries = foParams.entrySet().iterator();
         while (entries.hasNext()) {
            Map.Entry me = (Map.Entry) entries.next();
            key = (String) me.getKey();
            value = (String) me.getValue();
            content += "<item name='" + key + "'>" + value + "</item>";

         }
      }

      content += "</root>";

      return content;
   }
   
   
   /**
    * Method generate PDF file specialized (XSLFast) XSL-FO template and FOP utility
    * 
    * @param cont
    * @param userAgentProps
    * @param templatePath
    * @return PDF file content
    */
   public static byte[] xml2pdf(byte[] cont,
                         HashMap userAgentProps,
                         String templatePath) throws BaseException {
      ByteArrayInputStream bais = null;
      ByteArrayOutputStream baos = null;
      try {
         bais = new ByteArrayInputStream(cont);
         baos = new ByteArrayOutputStream();

         String fopRender = MimeConstants.MIME_PDF;

         FopFactory fopFactory = TransformUtility.getFopFactory();
         String fopBase = TransformUtility.getFopBase();

         FOUserAgent userAgent = fopFactory.newFOUserAgent();
         userAgent.setBaseURL("file:///" + fopBase + "/");

         userAgent.setRendererOverride(null);
         userAgent.setAuthor((String) userAgentProps.get("author"));
         userAgent.setCreationDate((Date) userAgentProps.get("date"));
         userAgent.setCreator((String) userAgentProps.get("creator"));
         userAgent.setProducer((String) userAgentProps.get("producer"));
         userAgent.setTitle((String) userAgentProps.get("title"));
         userAgent.setKeywords((String) userAgentProps.get("keywords"));
         userAgent.setTargetResolution(((Integer) userAgentProps.get("resolution")).intValue());

         userAgent.getRendererOptions()
            .put("encryption-params",
                 new PDFEncryptionParams((String) userAgentProps.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                                + EnhydraDMConstants.PDF_PARAM_USERPASSWORD),
                                         (String) userAgentProps.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                                + EnhydraDMConstants.PDF_PARAM_OWNERPASSWORD),
                                         ((Boolean) userAgentProps.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                                  + EnhydraDMConstants.PDF_PARAM_ALLOWPRINT)).booleanValue(),
                                         ((Boolean) userAgentProps.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                                  + EnhydraDMConstants.PDF_PARAM_ALLOWCOPYCONTENT)).booleanValue(),
                                         ((Boolean) userAgentProps.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                                  + EnhydraDMConstants.PDF_PARAM_ALLOWEDITANNOTATION)).booleanValue(),
                                         ((Boolean) userAgentProps.get(EnhydraDMConstants.PDF_PARAM_PREFIX
                                                                  + EnhydraDMConstants.PDF_PARAM_ALLOWEDITANNOTATION)).booleanValue()));

         Fop fop = fopFactory.newFop(fopRender, userAgent, baos);

         InputStream in = new FileInputStream(templatePath);
         Transformer foTransformer = transformerFactory.newTransformer(new StreamSource(in));

         Source src = new StreamSource(bais);
         Result res = new SAXResult(fop.getDefaultHandler());
         foTransformer.transform(src, res);
         cont = baos.toByteArray();
      } catch (Exception ex) {
         throw new BaseException(ex);
      } finally {
         if (bais != null)
            try {
               bais.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
         if (baos != null)
            try {
               baos.close();
            } catch (IOException e) {
               e.printStackTrace();
            }
      }
      return cont;
   }

}
